from osbot_aws.apis.EC2 import EC2
from osbot_utils.base_classes.Kwargs_To_Self import Kwargs_To_Self
from osbot_utils.helpers.SSH import SSH
from osbot_utils.utils.Dev import pprint
from osbot_utils.utils.Misc import random_string

from osbot_aws.helpers.AMI import AMI


class EC2_Instance(Kwargs_To_Self):
    instance_id   : str
    create_kwargs : dict
    ec2           : EC2
    # def __init__(self, instance_id=None, create_kwargs=None):
    #     self.create_kwargs = create_kwargs
    #     self.instance_id   = instance_id
    #     self.ec2           = EC2()

    def create(self):
        kwargs = self.create_kwargs
        random_name = random_string(prefix='test_ec2_with_ssh_support')
        if not kwargs.get('image_id'     ): kwargs['image_id'     ] = AMI().amazon_linux_2()
        if not kwargs.get('instance_type'): kwargs['instance_type'] = 't2.nano'
        if not kwargs.get('name'         ): kwargs['name'         ] = random_name
        if not kwargs.get('tags'         ): kwargs['tags'         ] = {'Type': 'EC2_Instance'}

        self.instance_id = self.ec2.instance_create(**kwargs)
        return self.instance_id

    def delete(self):
        return self.ec2.instance_delete(instance_id=self.instance_id)

    def exists(self):
        return self.ec2.instance_exists(instance_id=self.instance_id)

    def info(self):
        return self.ec2.instance_details(instance_id=self.instance_id)

    def print_info(self):
        pprint(self.info())

    def ip_address(self):
        return self.info().get('public_ip')

    def start(self):
        return self.ec2.instance_start(instance_id=self.instance_id)

    def stop(self):
        return self.ec2.instance_terminate(instance_id=self.instance_id)

    def ssh(self, ssh_key_file, ssh_key_user):
        ip_address = self.ip_address()
        return SSH(ssh_host=ip_address, ssh_key_file=ssh_key_file, ssh_key_user=ssh_key_user)

    def state(self):
        return self.info().get('state')

    def tags(self):
        return self.info().get('tags')


from osbot_aws.aws.sts.STS import STS
from osbot_utils.decorators.methods.cache_on_self import cache_on_self


class AMI:

    def __init__(self):
        self.current_region_name = self.sts().current_region_name()

    @cache_on_self
    def sts(self):
        return STS()

    @cache_on_self
    def ami_mappings_per_region(self):                                                          # todo find way to pre-create these in a performant way (for all regions)
        mappings = {'amazon-linux-2': { 'eu-west-1': 'ami-00f8b1192da5566c5' ,
                                        'eu-west-2': 'ami-074882b79a16e2e6e' }}
        return mappings

    def get_ami_for_image(self, image_name, region_name):
        return self.ami_mappings_per_region().get(image_name,{}).get(region_name,'')

    def amazon_linux_2(self):
        image_name = 'amazon-linux-2'
        return self.get_ami_for_image(image_name=image_name, region_name=self.current_region_name)


from unittest import TestCase

from dotenv import load_dotenv

from osbot_aws.aws.boto3.Cache_Boto3_Requests import Cache_Boto3_Requests


class TestCase__Boto3_Cache(TestCase):
    cache : Cache_Boto3_Requests

    @classmethod
    def setUpClass(cls):
        load_dotenv()
        cls.cache = Cache_Boto3_Requests()
        cls.cache.patch_apply()

    @classmethod
    def tearDownClass(cls):
        cls.cache.patch_restore()



from functools import wraps


class aws_retry:

    """retries the call (required for some cases where data is not consistent yet in AWS"""
    def __init__(self, fields):
        self.fields = fields                                        # field to inject

    def __call__(self, function):
        pass
        #code from aws_inject
        # from osbot_aws.AWS_Config import AWS_Config
        # @wraps(function)                                            # makes __name__ work ok
        # def wrapper(*args,**kwargs):                                # wrapper function
        #     for field in self.fields.split(','):                    # split value provided by comma
        #         if field == 'region'    : kwargs[field] = AWS_Config().aws_session_region_name()
        #         if field == 'account_id': kwargs[field] = AWS_Config().aws_session_account_id()
        #     return function(*args,**kwargs)
        #return wrapper


from botocore.client                                                    import BaseClient
from botocore.exceptions                                                import ClientError
from osbot_utils.helpers.sqlite.domains.Sqlite__Cache__Requests__Patch  import Sqlite__Cache__Requests__Patch
from osbot_utils.utils.Yaml                                             import yaml_to_str

SQLITE_DB_NAME__BOTO3_REQUESTS_CACHE = 'boto3_requests_cache.sqlite'
SQLITE_TABLE_NAME__BOTO3_REQUESTS    = 'boto3_requests'

class Cache_Boto3_Requests(Sqlite__Cache__Requests__Patch):

    def __init__(self, db_path=None):
        self.target_class           = BaseClient
        self.target_function        = BaseClient._make_api_call
        self.target_function_name   = "_make_api_call"
        self.db_name                = SQLITE_DB_NAME__BOTO3_REQUESTS_CACHE
        self.table_name             = SQLITE_TABLE_NAME__BOTO3_REQUESTS
        self.capture_exceptions     = True
        self.exception_classes      = [ClientError]
        #self.print_requests         = False
        super().__init__(db_path=db_path)

    def invoke_target(self, target, target_args, target_kwargs):
        #if self.print_requests:
            #print(f'[invoke_target]: {target_args}')
        return super().invoke_target(target, target_args, target_kwargs)

    def request_data(self, *args, **kwargs):
        target_self, operation_name, api_params = args
        request_data =  {'operation_name': operation_name,
                        'api_params'    : api_params    }
        #if self.print_requests:
            #print(f'[request_data]: {request_data}')
        request_data = yaml_to_str(request_data)
        return request_data